/*****************************************************************************
 *
 * evemu - Kernel device emulation
 *
 * Copyright (C) 2011 Red Hat, Inc.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "evemu.h"
#include <limits.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/input.h>

int main(int argc, char *argv[])
{
	int rc = -1;
	int fd;
	long int type, code, value;
	struct input_event ev;
	int idx = 1;
	int sync = 0;
	const char *path;
	const char *arg;
	char *endp;

	if (argc < 5) {
		fprintf(stderr, "Usage: %s [--sync] <device> <type> <code> <value>\n", argv[0]);
		goto out;
	}

	if (!strcmp(argv[1], "--sync")) {
		idx = 2;
		sync = 1;
	}

	path = argv[idx++];

	arg = argv[idx++];
	type = strtol(arg, &endp, 0);
	if (*arg == '\0' || *endp != '\0' || type < 0 || type > EV_MAX) {
		fprintf(stderr, "error: invalid type argument '%s'\n", arg);
		goto out;
	}

	arg = argv[idx++];
	code = strtol(arg, &endp, 0);
	if (*arg == '\0' || *endp != '\0' || code < 0 || code > USHRT_MAX) {
		fprintf(stderr, "error: invalid code argument '%s'\n", arg);
		goto out;
	}

	arg = argv[idx++];
	value = strtol(arg, &endp, 0);
	if (*arg == '\0' || *endp != '\0' || value < INT_MIN || value > INT_MAX) {
		fprintf(stderr, "error: invalid value argument '%s'\n", arg);
		goto out;
	}

	fd = open(path, O_WRONLY);
	if (fd < 0) {
		fprintf(stderr, "error: could not open device\n");
		goto out;
	}

	if (evemu_create_event(&ev, type, code, value)) {
		fprintf(stderr, "error: failed to create event\n");
		goto out;
	}

	if (evemu_play_one(fd, &ev)) {
		fprintf(stderr, "error: could not play event\n");
		goto out;
	}

	if (sync) {
		if (evemu_create_event(&ev, EV_SYN, SYN_REPORT, 0)) {
			fprintf(stderr, "error: could not create SYN event\n");
			goto out;
		}
		if (evemu_play_one(fd, &ev)) {
			fprintf(stderr, "error: could not play SYN event\n");
			goto out;
		}
	}

	rc = 0;
out:
	close(fd);
	return rc;
}
